@@ -7,21 +7,25 @@ For usage see
 - :py:class:`Client` class
 
 """
-
+import datetime
+import logging
 import os
 import tempfile
 import time
+import warnings
 from functools import wraps
 from json import JSONDecodeError
-from typing import Optional
 from pathlib import Path
-import logging
+from typing import Final, Optional, Set, Union
 
 # TODO: Must be here because  warnings are very inconveniently triggered import time
+import pandas as pd
 from tqdm import TqdmExperimentalWarning
-import warnings
+
 # "Using `tqdm.autonotebook.tqdm` in notebook mode. Use `tqdm.tqdm` instead to force console mode (e.g. in jupyter console) from tqdm.autonotebook import tqdm"
-from tradingstrategy.universe import Universe
+from tradingstrategy.reader import BrokenData, read_parquet
+from tradingstrategy.transport.pyodide import PYODIDE_API_KEY
+from tradingstrategy.utils.jupyter import is_pyodide
 
 warnings.filterwarnings("ignore", category=TqdmExperimentalWarning)
 
@@ -38,19 +42,26 @@ import pyarrow
 import pyarrow as pa
 from pyarrow import Table
 
-from tradingstrategy.timebucket import TimeBucket
-from tradingstrategy.environment.config import Configuration
-from tradingstrategy.exchange import ExchangeUniverse
-from tradingstrategy.reader import read_parquet, BrokenData
 from tradingstrategy.chain import ChainId
 from tradingstrategy.environment.base import Environment, download_with_progress_plain
-from tradingstrategy.environment.jupyter import JupyterEnvironment, download_with_tqdm_progress_bar
+from tradingstrategy.environment.config import Configuration
+from tradingstrategy.environment.jupyter import (
+    JupyterEnvironment,
+    download_with_tqdm_progress_bar,
+)
+from tradingstrategy.exchange import ExchangeUniverse
+from tradingstrategy.timebucket import TimeBucket
 from tradingstrategy.transport.cache import CachedHTTPTransport
 
 
 logger = logging.getLogger(__name__)
 
 
+RETRY_DELAY: Final[int] = 30  # seconds
+
+MAX_ATTEMPTS: Final[int] = 3
+
+
 def _retry_corrupted_parquet_fetch(method):
     """A helper decorator to down with download/Parquet corruption issues.
 
@@ -59,7 +70,7 @@ def _retry_corrupted_parquet_fetch(method):
     # https://stackoverflow.com/a/36944992/315168
     @wraps(method)
     def impl(self, *method_args, **method_kwargs):
-        attempts = 3
+        attempts = MAX_ATTEMPTS
         while attempts > 0:
             try:
                 return method(self, *method_args, **method_kwargs)
@@ -68,14 +79,25 @@ def _retry_corrupted_parquet_fetch(method):
             except (OSError, BrokenData) as e:
                 # This happens when we download Parquet file, but it is missing half
                 # e.g. due to interrupted download
+                attempts -= 1
+                path_to_remove = e.path if isinstance(e, BrokenData) else None
+
                 if attempts > 0:
                     logger.error("Damaged Parquet file fetch detected for method %s, attempting to re-fetch. Error was: %s", method, e)
                     logger.exception(e)
-                    # TODO: Only clear the damaged file, not all caches
-                    self.clear_caches()
-                    attempts -= 1
-                    time.sleep(30)
+
+                    self.clear_caches(filename=path_to_remove)
+
+                    logger.info(
+                        f"Next parquet download retry in {RETRY_DELAY} seconds, "
+                        f"{attempts}/{MAX_ATTEMPTS} attempt(s) left"
+                    )
+                    time.sleep(RETRY_DELAY)
                 else:
+                    logger.warning(
+                        f"Exhausted all {MAX_ATTEMPTS} attempts, fetching parquet data failed."
+                    )
+                    self.clear_caches(filename=path_to_remove)
                     raise
 
     return impl
@@ -120,12 +142,15 @@ class Client:
         self.env = env
         self.transport = transport
 
-    def clear_caches(self):
+    def clear_caches(self, filename: Optional[Union[str, Path]] = None):
         """Remove any cached data.
 
         Cache is specific to the current transport.
+
+        :param filename:
+            If given, remove only that specific file, otherwise clear all cached data.
         """
-        self.transport.purge_cache()
+        self.transport.purge_cache(filename)
 
     @_retry_corrupted_parquet_fetch
     def fetch_pair_universe(self) -> pa.Table:
@@ -164,6 +189,64 @@ class Client:
         path = self.transport.fetch_candles_all_time(bucket)
         return read_parquet(path)
 
+    def fetch_candles_by_pair_ids(self,
+          pair_ids: Set[int],
+          bucket: TimeBucket,
+          start_time: Optional[datetime.datetime] = None,
+          end_time: Optional[datetime.datetime] = None,
+          max_bytes: Optional[int] = None,
+          progress_bar_description: Optional[str] = None,
+        ) -> pd.DataFrame:
+        """Fetch candles for particular trading pairs.
+
+        The fetch is performed using JSONL API endpoint. If the number
+        of trading pair is small, this download is much more lightweight
+        than Parquet dataset download.
+
+        :param pair_ids:
+            Trading pairs internal ids we query data for.
+            Get internal ids from pair dataset.
+
+        :param time_bucket:
+            Candle time frame
+
+        :param start_time:
+            All candles after this.
+            If not given start from genesis.
+
+        :param end_time:
+            All candles before this
+
+        :param max_bytes:
+            Limit the streaming response size
+
+        :param progress_bar_description:
+            Display on download progress bar
+
+        :return:
+            Candles dataframe
+
+        :raise tradingstrategy.transport.jsonl.JSONLMaxResponseSizeExceeded:
+                If the max_bytes limit is breached
+        """
+        return self.transport.fetch_candles_by_pair_ids(
+            pair_ids,
+            bucket,
+            start_time,
+            end_time,
+            max_bytes=max_bytes,
+            progress_bar_description=progress_bar_description,
+        )
+
+    def fetch_candle_dataset(self, bucket: TimeBucket) -> Path:
+        """Fetch candle data from the server.
+
+        Do not attempt to decode the Parquet file to the memory,
+        but instead of return raw
+        """
+        path = self.transport.fetch_candles_all_time(bucket)
+        return path
+
     @_retry_corrupted_parquet_fetch
     def fetch_all_liquidity_samples(self, bucket: TimeBucket) -> Table:
         """Get cached blob of liquidity events of a certain time window.
@@ -190,22 +273,11 @@ class Client:
 
         # Work around Google Colab shipping with old Pandas
         # https://stackoverflow.com/questions/11887762/how-do-i-compare-version-numbers-in-python
-        from packaging import version
         import pandas
+        from packaging import version
         pandas_version = version.parse(pandas.__version__)
         assert pandas_version >= version.parse("1.3"), f"Pandas 1.3.0 or greater is needed. You have {pandas.__version__}. If you are running this notebook in Google Colab and this is the first run, you need to choose Runtime > Restart and run all from the menu to force the server to load newly installed version of Pandas library."
 
-        # Fix Backtrader / Pandas 1.3 issue that breaks FastQuant
-        try:
-            import fastquant
-            fastquant_enabled = True
-        except ImportError:
-            fastquant_enabled = False
-
-        if fastquant_enabled:
-            from tradingstrategy.frameworks.fastquant_monkey_patch import apply_patch
-            apply_patch()
-
     @classmethod
     def setup_notebook(cls):
         """Setup diagram rendering and such.
@@ -217,16 +289,94 @@ class Client:
         mpl.rcParams['figure.dpi'] = 600
 
     @classmethod
-    def create_jupyter_client(cls, cache_path: Optional[str]=None, api_key: Optional[str]=None) -> "Client":
+    async def create_pyodide_client_async(cls,
+                                    cache_path: Optional[str] = None,
+                                    api_key: Optional[str] = PYODIDE_API_KEY,
+                                    remember_key=False) -> "Client":
+        """Create a new API client inside Pyodide enviroment.
+
+        `More information about Pyodide project / running Python in a browser <https://pyodide.org/>`_.
+
+        :param cache_path:
+            Virtual file system path
+
+        :param cache_api_key:
+            The API key used with the server downloads.
+            A special hardcoded API key is used to identify Pyodide
+            client and its XmlHttpRequests. A referral
+            check for these requests is performed.
+
+        :param remember_key:
+            Store the API key in IndexDB for the future use
+
+        :return:
+            pass
+        """
+        from tradingstrategy.environment.jupyterlite import IndexDB
+
+        # Store API
+        if remember_key:
+
+            db = IndexDB()
+
+            if api_key:
+                await db.set_file("api_key", api_key)
+
+            else:
+                api_key = await db.get_file("api_key")
+
+        return cls.create_jupyter_client(cache_path, api_key, pyodide=True)
+
+    @classmethod
+    def create_jupyter_client(cls,
+                              cache_path: Optional[str] = None,
+                              api_key: Optional[str] = None,
+                              pyodide=None,
+                              ) -> "Client":
         """Create a new API client.
 
-        :param cache_path: Where downloaded datasets are stored. Defaults to `~/.cache`.
+        This function is intented to be used from Jupyter notebooks
+
+        - Any local or server-side IPython session
+
+        - JupyterLite notebooks
+
+        :param api_key:
+            If not given, do an interactive API key set up in the Jupyter notebook
+            while it is being run.
+
+        :param cache_path:
+            Where downloaded datasets are stored. Defaults to `~/.cache`.
+
+        :param pyodide:
+            Detect the use of this library inside Pyodide / JupyterLite.
+            If `None` then autodetect Pyodide presence,
+            otherwise can be forced with `True`.
         """
+
+        if pyodide is None:
+            pyodide = is_pyodide()
+
         cls.preflight_check()
         cls.setup_notebook()
         env = JupyterEnvironment()
-        config = env.setup_on_demand()
-        transport = CachedHTTPTransport(download_with_tqdm_progress_bar, cache_path=env.get_cache_path(), api_key=config.api_key)
+
+        # Try Pyodide default key
+        if not api_key:
+            if pyodide:
+                api_key = PYODIDE_API_KEY
+
+        # Try file system stored API key,
+        # if not prompt interactively
+        if not api_key:
+            config = env.setup_on_demand(api_key=api_key)
+            api_key = config.api_key
+
+        cache_path = cache_path or env.get_cache_path()
+        transport = CachedHTTPTransport(
+            download_with_tqdm_progress_bar,
+            cache_path=cache_path,
+            api_key=api_key)
         return Client(env, transport)
 
     @classmethod
