@@ -1,16 +1,23 @@
 import functools
-import os
-import shutil
-from typing import Optional
 import logging
+import os
+import platform
+import shutil
+import sys
+from typing import Optional
 
 from requests import Session
-from tqdm.autonotebook import tqdm
-
+from tqdm.auto import tqdm
 from tradingstrategy.environment.base import Environment
 from tradingstrategy.environment.config import Configuration
-from tradingstrategy.environment.interactive_setup import run_interactive_setup
+from tradingstrategy.environment.interactive_setup import (
+    run_interactive_setup,
+    run_non_interactive_setup,
+)
 
+if platform.system() == 'Emscripten':
+    # disable tqdm thread in pyodide - it doesn't have threading yet
+    tqdm.monitor_interval = 0
 
 logger = logging.getLogger(__name__)
 
@@ -49,18 +56,36 @@ class JupyterEnvironment(Environment):
             data = config.to_json()
             out.write(data)
 
+    def clear_configuration(self):
+        """Delete the saved config file (if any)"""
+        spath = self.get_settings_path()
+        path = os.path.join(spath, "settings.json")
+        if os.path.exists(path):
+            os.remove(path)
+
     def interactive_setup(self) -> Configuration:
         """Perform interactive user onbaording"""
         config = run_interactive_setup()
         self.save_configuration(config)
         return config
 
-    def setup_on_demand(self) -> Configuration:
+    def non_interactive_setup(self, **kwargs) -> Configuration:
+        """Perform interactive user onbaording"""
+        config = run_non_interactive_setup(**kwargs)
+        if config:
+            self.save_configuration(config)
+        return config
+
+    def setup_on_demand(self, **kwargs) -> Configuration:
         """Check if we need to set up the environment."""
         config = self.discover_configuration()
         if not config:
-            print(f"No existing Trading Strategy configuration found in {self.get_settings_path()}/settings.json. Starting interactive setup.")
-            config = self.interactive_setup()
+            if platform.system() == 'Emscripten':
+                print(f"No existing Trading Strategy configuration found in {self.get_settings_path()}/settings.json. Making config from keyword parameters.")
+                config = self.non_interactive_setup(**kwargs)
+            else:
+                print(f"No existing Trading Strategy configuration found in {self.get_settings_path()}/settings.json. Starting interactive setup.")
+                config = self.interactive_setup()
         else:
             print(f"Started Trading Strategy in Jupyter notebook environment, configuration is stored in {self.get_settings_path()}")
         return config
