@@ -186,7 +186,7 @@ class ExchangeUniverse:
         name = name.lower()
         assert isinstance(chain_id, ChainId)
         for xchg in self.exchanges.values():
-            if xchg.name.lower() == name:
+            if xchg.name.lower() == name.lower():
                 return xchg
         return None
 
@@ -216,3 +216,4 @@ class ExchangeUniverse:
             if xchg.address.lower() == factory_address:
                 return xchg
         return None
+
