@@ -18,6 +18,7 @@ from dataclasses import dataclass
 from typing import Optional, List, Iterable, Dict, Union, Set, Tuple
 
 import pandas as pd
+import numpy as np
 import pyarrow as pa
 import pyarrow.compute as pc
 from dataclasses_json import dataclass_json
@@ -448,8 +449,9 @@ class PandasPairUniverse:
         assert isinstance(df, pd.DataFrame)
         self.df = df.set_index(df["pair_id"])
 
-        # pair_id -> DEXPair
-        self.pair_map: Dict[int, DEXPair] = {}
+        #: pair_id -> DEXPair mappings
+        #: Don't access directly, use :py:meth:`iterate_pairs`.
+        self.pair_map: Dict[int, dict] = {}
 
         # pair smart contract address -> DEXPair
         self.smart_contract_map = {}
@@ -461,15 +463,28 @@ class PandasPairUniverse:
         if build_index:
             self.build_index()
 
+    def iterate_pairs(self) -> Iterable[DEXPair]:
+        """Iterate over all pairs in this universe."""
+        for p in self.pair_map.values():
+            yield DEXPair.from_dict(p)
+
     def build_index(self):
         """Create pair_id -> data mapping.
 
         Allows fast lookup of individual pairs.
+
+        .. warning::
+
+            This function assumes the universe contains
+            data for only one blockchain. The same address
+            can exist across multiple EVM chains.
+            The created smart contract address index
+            does not index chain id and thus is invalid.
+
         """
-        for pair_id, data in self.df.iterrows():
-            pair: DEXPair = DEXPair.from_dict(data)
-            self.pair_map[pair_id] = pair
-            self.smart_contract_map[pair.address.lower()] = pair
+        # https://stackoverflow.com/a/73638890/315168
+        self.pair_map = self.df.T.to_dict()
+        self.smart_contract_map = {d["address"].lower(): d for d in self.pair_map.values()}
 
     def get_all_pair_ids(self) -> List[PrimaryKey]:
         return self.df["pair_id"].unique()
@@ -486,7 +501,8 @@ class PandasPairUniverse:
 
         if self.pair_map:
             # TODO: Eliminate non-indexed code path?
-            return self.pair_map.get(pair_id)
+            data = self.pair_map.get(pair_id)
+            return DEXPair.from_dict(data)
 
         # TODO: Remove
 
@@ -506,11 +522,18 @@ class PandasPairUniverse:
     def get_pair_by_smart_contract(self, address: str) -> Optional[DEXPair]:
         """Resolve a trading pair by its pool smart contract address.
 
+        .. warning::
+
+            This function assumes the universe contains
+            data for only one blockchain. The same address
+            can exist across multiple EVM chains.
+
         :param address: Ethereum smart contract address of the Uniswap pair contract
         """
         address = address.lower()
         assert self.smart_contract_map, "You need to build the index to use this function"
-        return self.smart_contract_map.get(address)
+        data = self.smart_contract_map.get(address)
+        return DEXPair.from_dict(data)
 
     def get_token(self, address: str) -> Optional[Token]:
         """Get a token that is part of any trade pair.
@@ -524,19 +547,20 @@ class PandasPairUniverse:
         :return:
             Tuple (name, symbol, address, decimals)
             or None if not found.
-
         """
         address = address.lower()
 
         token: Optional[Token] = None
 
         if address not in self.token_cache:
-            for p in self.pair_map.values():
-                if p.quote_token_address == address:
-                    token = Token(p.chain_id, p.quote_token_symbol, address, p.quote_token_decimals)
+            for pair_data in self.pair_map.values():
+                if pair_data["token0_address"] == address:
+                    p = DEXPair.from_dict(pair_data)
+                    token = Token(p.chain_id, p.token0_address, address, p.token0_decimals)
                     break
-                elif p.base_token_address == address:
-                    token = Token(p.chain_id, p.base_token_symbol, address, p.base_token_decimals)
+                elif pair_data["token1_address"] == address:
+                    p = DEXPair.from_dict(pair_data)
+                    token = Token(p.chain_id, p.token1_address, address, p.token1_decimals)
                     break
             self.token_cache[address] = token
         return self.token_cache[address]
@@ -551,7 +575,8 @@ class PandasPairUniverse:
 
         """
         tokens = set()
-        for p in self.pair_map.values():
+        for pair_data in self.pair_map.values():
+            p = DEXPair.from_dict(pair_data)
             tokens.add(self.get_token(p.base_token_address))
             tokens.add(self.get_token(p.quote_token_address))
         return tokens
@@ -563,7 +588,7 @@ class PandasPairUniverse:
         """
         pair_count = len(self.pair_map)
         assert pair_count == 1, f"Not a single trading pair universe, we have {pair_count} pairs"
-        return next(iter(self.pair_map.values()))
+        return DEXPair.from_dict(next(iter(self.pair_map.values())))
 
     def get_by_symbols(self, base_token_symbol: str, quote_token_symbol: str) -> Optional[DEXPair]:
         """For strategies that trade only a few trading pairs, get the only pair in the universe.
@@ -576,8 +601,8 @@ class PandasPairUniverse:
 
         """
         for pair in self.pair_map.values():
-            if pair.base_token_symbol == base_token_symbol and pair.quote_token_symbol == quote_token_symbol:
-                return pair
+            if pair["base_token_symbol"] == base_token_symbol and pair["quote_token_symbol"] == quote_token_symbol:
+                return DEXPair.from_dict(pair)
         return None
 
     def get_one_pair_from_pandas_universe(self, exchange_id: PrimaryKey, base_token: str, quote_token: str, pick_by_highest_vol=False) -> Optional[DEXPair]:
@@ -638,6 +663,28 @@ class PandasPairUniverse:
 
         return None
 
+    def create_parquet_load_filter(self, count_limit=10_000) -> List[Tuple]:
+        """Returns a Parquet loading filter that contains pairs in this universe.
+
+        When candle or liquidity file is read to the memory,
+        only read pairs that are within this pair universe.
+        This severely reduces the memory usage and speed ups loading.
+
+        See :py:func:`tradingstrategy.reader.read_parquet`.
+
+        :param count_limit:
+            Sanity check assert limit how many pairs we can cram into the filter.
+
+        :return:
+            Filter to be passed to read_table
+        """
+
+        count = self.get_count()
+        assert count < count_limit, f"Too many pairs to create a filter. Pair count is {count}"
+
+        # https://arrow.apache.org/docs/python/generated/pyarrow.parquet.read_table.html
+        return [("pair_id", "in", self.get_all_pair_ids())]
+
     @staticmethod
     def create_single_pair_universe(
             df: pd.DataFrame,
@@ -944,4 +991,101 @@ def filter_for_stablecoins(pairs: pd.DataFrame, mode: StablecoinFilteringMode) -
         our_pairs: pd.DataFrame = pairs.loc[
             ~(pairs['token0_symbol'].isin(ALL_STABLECOIN_LIKE) & pairs['token1_symbol'].isin(ALL_STABLECOIN_LIKE))
         ]
-    return our_pairs
\ No newline at end of file
+    return our_pairs
+
+def resolve_pairs_based_on_ticker(
+        df: pd.DataFrame,
+        chain_id: ChainId,
+        exchange_slug: str,
+        pair_tickers: Set[Tuple[str, str]],
+) -> pd.DataFrame:
+    """Resolve symbolic trading pairs to their internal integer primary key ids.
+
+    Uses pair database described :py:class:`DEXPair` Pandas dataframe
+    to resolve pairs to their integer ids on a single exchange.
+
+    .. warning ::
+
+        For popular trading pairs, there will be multiple scam pairs
+        with the same ticker name. In this case, one with the highest all-time
+        buy volume is chosen.
+
+    .. note ::
+
+        Pair ids are not stable and may change long term.
+        Always resolve pair ids before a run.
+
+    Example:
+
+    .. code-block: python
+
+        pairs_df = client.fetch_pair_universe().to_pandas()
+
+        tickers = {
+            ("WBNB", "BUSD"),
+            ("Cake", "WBNB"),
+        }
+
+        # ticker -> pd.Series row map for pairs
+        filtered_pairs_df = resolve_pairs_based_on_ticker(
+            pairs_df,
+            ChainId.bsc,
+            "pancakeswap-v2",
+            tickers
+        )
+
+        assert len(filtered_pairs_df) == 2
+        wbnb_busd = filtered_pairs_df.loc[
+            (filtered_pairs_df["base_token_symbol"] == "WBNB") &
+            (filtered_pairs_df["quote_token_symbol"] == "BUSD")
+        ].iloc[0]
+        assert wbnb_busd["buy_volume_30d"] > 0
+
+    :param df:
+        DataFrame containing DEXPairs
+
+    :param chain_id:
+        Blockchain the exchange is on
+
+    :param exchange_slug:
+        Symbolic exchange name
+
+    :param pair_tickers:
+        List of trading pairs as (base token, quote token) tuples.
+        Note that giving trading pair tokens in wrong order
+        causes pairs not to be found.
+        If any ticker does not match it is not included in the result set.
+
+    :return:
+        DataFrame with filtered pairs.
+    """
+
+    # Create list of conditions to filter out dataframe,
+    # one condition per pair
+    conditions = []
+    for base, quote in pair_tickers:
+        condition = (df["base_token_symbol"].str.lower() == base.lower()) & \
+                    (df["quote_token_symbol"].str.lower() == quote.lower())
+        conditions.append(condition)
+
+    # OR call conditions together
+    # https://stackoverflow.com/a/57468610/315168
+    df_matches = df.loc[np.logical_or.reduce(conditions)]
+
+    # Sort by the buy volume as a preparation
+    # for the scam filter
+    df_matches = df_matches.sort_values(by="buy_volume_all_time", ascending=False)
+
+    result_pair_ids = set()
+
+    # Scam filter.
+    # Pick the tokens by the highest buy volume to the result map.
+    for test_pair in pair_tickers:
+        for idx, row in df_matches.iterrows():
+            if row["base_token_symbol"] == test_pair[0] and \
+               row["quote_token_symbol"] == test_pair[1]:
+                    result_pair_ids.add(row["pair_id"])
+                    break
+
+    result_df = df.loc[df["pair_id"].isin(result_pair_ids)]
+    return result_df
