@@ -1,15 +1,19 @@
 import datetime
-import io
+import hashlib
 import os
 import pathlib
-from typing import Optional, Callable
+import re
+from typing import Optional, Callable, Set, Union
 import shutil
 import logging
 
+import pandas
+import pandas as pd
 import requests
 from requests import Response
 
 from tradingstrategy.timebucket import TimeBucket
+from tradingstrategy.transport.jsonl import load_candles_jsonl
 
 logger = logging.getLogger(__name__)
 
@@ -89,13 +93,19 @@ class CachedHTTPTransport:
         path = os.path.join(self.get_abs_cache_path(), fname)
         return path
 
-    def get_cached_item(self, fname) -> Optional[pathlib.Path]:
+    def get_cached_item(self, fname: Union[str, pathlib.Path]) -> Optional[pathlib.Path]:
 
         path = self.get_cached_file_path(fname)
         if not os.path.exists(path):
             return None
 
         f = pathlib.Path(path)
+
+        end_time_pattern = r"-to_\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}"
+        if re.search(end_time_pattern, str(fname)):
+            # Candle files with an end time never expire, as the history does not change
+            return f
+
         mtime = datetime.datetime.fromtimestamp(f.stat().st_mtime)
         if datetime.datetime.now() - mtime > self.cache_period:
             # File cache expired
@@ -103,11 +113,56 @@ class CachedHTTPTransport:
 
         return f
 
-    def purge_cache(self):
-        """Delete all cached files on the filesystem."""
-        if os.path.exists(self.cache_path):
-            logger.info("Purging caches at %s", self.cache_path)
-            shutil.rmtree(self.cache_path)
+    def _generate_cache_name(
+        self,
+        pair_ids: Set[id],
+        time_bucket: TimeBucket,
+        start_time: Optional[datetime.datetime] = None,
+        end_time: Optional[datetime.datetime] = None,
+        max_bytes: Optional[int] = None,
+    ) -> str:
+        """Generate the name of the file for holding cached candle data for ``pair_ids``.
+        """
+        # Meaningfully truncate the timestamp to align with the target time bucket.
+        if end_time:
+            candle_width = time_bucket.to_timedelta()
+            trunc = {"second": 0}
+            if candle_width >= datetime.timedelta(hours=1):
+                trunc["minute"] = 0
+            if candle_width >= datetime.timedelta(days=1):
+                trunc["hour"] = 0
+            end_time = end_time.replace(**trunc)
+
+        # Create a compressed cache key for the filename,
+        # as we have 256 char limit on fname lenghts
+        full_cache_key = (
+            f"{pair_ids}{time_bucket}{start_time}{end_time}{max_bytes}"
+        )
+        md5 = hashlib.md5(full_cache_key.encode("utf-8")).hexdigest()
+
+        # If exists, include the end time info in filename for cache invalidation logic.
+        end_part = end_time.strftime("-to_%Y-%m-%d_%H-%M-%S") if end_time else ""
+
+        return f"candles-jsonl-{time_bucket.value}{end_part}-{md5}.parquet"
+
+    def purge_cache(self, filename: Optional[Union[str, pathlib.Path]] = None):
+        """Delete all cached files on the filesystem.
+
+        :param filename:
+            If given, remove only that specific file, otherwise clear all cached data.
+        """
+        target_path = self.cache_path if filename is None else filename
+
+        logger.info("Purging caches at %s", target_path)
+        try:
+            if os.path.isdir(target_path):
+                shutil.rmtree(target_path)
+            else:
+                os.remove(target_path)
+        except FileNotFoundError as exc:
+            logger.warning(
+                f"Attempted to purge caches, but no such file or directory: {exc.filename}"
+            )
 
     def save_response(self, fpath, api_path, params=None, human_readable_hint: Optional[str]=None):
         """Download a file to the cache and display a pretty progress bar while doing it.
@@ -203,3 +258,73 @@ class CachedHTTPTransport:
         """
         reply = self.post_json_response("register", params={"first_name": first_name, "last_name": last_name, "email": email})
         return reply
+
+    def fetch_candles_by_pair_ids(
+            self,
+            pair_ids: Set[id],
+            time_bucket: TimeBucket,
+            start_time: Optional[datetime.datetime] = None,
+            end_time: Optional[datetime.datetime] = None,
+            max_bytes: Optional[int] = None,
+            progress_bar_description: Optional[str] = None,
+    ) -> pd.DataFrame:
+        """Load particular set of the candles and cache the result.
+
+        If there is no cached result, load using JSONL.
+
+        More information in :py:mod:`tradingstrategy.transport.jsonl`.
+
+        For the candles format see :py:mod:`tradingstrategy.candle`.
+
+        :param pair_ids:
+            Trading pairs internal ids we query data for.
+            Get internal ids from pair dataset.
+
+        :param time_bucket:
+            Candle time frame
+
+        :param start_time:
+            All candles after this.
+            If not given start from genesis.
+
+        :param end_time:
+            All candles before this
+
+        :param max_bytes:
+            Limit the streaming response size
+
+        :param progress_bar_description:
+            Display on downlood progress bar
+
+        :return:
+            Candles dataframe
+        """
+        cache_fname = self._generate_cache_name(
+            pair_ids, time_bucket, start_time, end_time, max_bytes
+        )
+        cached = self.get_cached_item(cache_fname)
+
+        if cached:
+            full_fname = self.get_cached_file_path(cache_fname)
+            logger.debug("Using cached JSONL data file %s", full_fname)
+            return pandas.read_parquet(cached)
+
+        df: pd.DataFrame = load_candles_jsonl(
+            self.requests,
+            self.endpoint,
+            pair_ids,
+            time_bucket,
+            start_time,
+            end_time,
+            max_bytes=max_bytes,
+            progress_bar_description=progress_bar_description,
+        )
+
+        # Update cache
+        path = self.get_cached_file_path(cache_fname)
+        df.to_parquet(path)
+
+        size = pathlib.Path(path).stat().st_size
+        logger.debug(f"Wrote {cache_fname}, disk size is {size:,}b")
+
+        return df
